; TESTLEDS.ASM 
;
; Original code by David Tait. Modified by Tony Hunt.
; connect four LEDs from each of RB0-RB3 to ground 
; via four 470 ohm resistors.  The LEDs are illuminated one at time in 
; a to-and-fro pattern. Input on RA0 switches the output sequence between
; to and fro and spread patterning.
;
;  program is set up for an RC oscillator. 
; The program can be used unchanged on any 16X8X device.

        LIST            P=16C84
        ERRORLEVEL      -302    ;SUPPRESS BANK SELECTION MESSAGES
        __CONFIG        3FF7H   ;RC OSC, WATCHDOG
        __IDLOCS        1234
;
PORTA   EQU     5    
PORTB   EQU     6
TRISA   EQU     85H 
TRISB   EQU     86H
OPTREG  EQU     81H
STATUS  EQU     3
CARRY   EQU     0
RP0     EQU     5
MSB     EQU     3               ;BIT POSITION OF LEFTMOST LED
IN      EQU     1
;
        CLRF    PORTB           ;ALL LEDS OFF
        BSF     STATUS,RP0      ;SELECT REGISTER BANK 1   
        BSF     TRISA,IN        ;SET PORTA BIT0 TO INPUT
        CLRF    TRISB           ;SET PORTB TO ALL OUTPUTS
        MOVLW   0AH
        MOVWF   OPTREG          ;ASSIGN PRESCALER (1:4) TO WDT
        BCF     STATUS,RP0      ;SELECT REGISTER BANK 0
GO      INCF    PORTB,F         ;TURN ON RIGHTMOST LED
        BCF     STATUS,CARRY    ;CLEAR CARRY
LEFT    SLEEP                   ;WAIT FOR WDT TIMEOUT
        RLF     PORTB,F         ;TURN ON LED TO LEFT
        BTFSS   PORTB,MSB       ;REACHED LEFTMOST?
        GOTO    LEFT            ;LOOP IF NOT
RIGHT   SLEEP                   ;WAIT FOR WDT TIMEOUT
        RRF     PORTB,F         ;TURN ON LED TO RIGHT
        BTFSS   PORTB,0         ;REACHED RIGHTMOST?
        GOTO    RIGHT           ;LOOP IF NOT
        BTFSC   PORTA,0         ;TEST FOR INPUT- SKIP IF NONE
        GOTO    SPREAD          ;SPREADING LEDS ROUTINE
        GOTO    LEFT            ;START NEW CYCLE
SPREAD  CLRF    PORTB           ;ALL LEDS OFF
        MOVLW   06H             ;2 CENTRE LEDS ON ONLY
        MOVWF   PORTB
        SLEEP
        SLEEP
        MOVLW   09H             ;2 OUTSIDE LEDS ON ONLY
        MOVWF   PORTB
        SLEEP
        SLEEP
        BTFSC   PORTA,0         ;LOOK FOR CHANGE ON INPUT
        GOTO    SPREAD
        CLRF    PORTB           ;ALL LEDS OFF AGAIN READY TO
        GOTO    GO              ;RESTART TO AND FRO OPERATIONS
;
        ORG     2100H
;
        DE      "Copyright (C) 1996 David Tait"
        END
